<template>

  <div>
    <div
      v-show="!readyToConfirm"
      id="personal-check"
      class="input-transaction form">
      <form
        id="transactionForm"
        action=""
        method="post">
        <p>Input {{ transactionType }} info below:</p>
        <input
          type="hidden"
          name="TransactionType"
          value="transactionType" >
        <ul class="errors"/>
        <div class="section">
          <label for="AccountNumber">Account Number<input
            ref="accountNumber"
            v-model="account"
            type="text"
            name="AccountNumber"
            @keypress.enter="validateTransactionInfo"></label>
          <label
            v-show="transactionType == 'Personal Check' || transactionType == 'Business Check'"
            for="CheckNumber">Check Number<input
              ref="checkNumber"
              v-model="checkNumber"
              type="text"
              name="CheckNumber"
              @keypress.enter="validateTransactionInfo"></label>
          <label for="Amount">Amount<input
            ref="amount"
            v-model="amount"
            type="text"
            name="Amount"
            value=""
            @keypress.enter="validateTransactionInfo"></label>
        </div>

        <div class="section">
          <input
            type="button"
            class="submit money"
            value="Submit"
            @click="validateTransactionInfo" >
        </div>

        <div class="clearfix"/>

        <div id="or">
          <p>&#8212; or &#8212;</p>
          <p>Choose a different transaction type:</p>
        </div>

        <ul class="homebuttongroup">
          <li><a
            class="shopbuttons personal"
            @click="parentInputType(1); clearTransaction();">Personal Check</a></li>
          <li><a
            class="shopbuttons personal"
            @click="parentInputType(2); clearTransaction();">Personal Deposit</a></li>
          <li><a
            class="shopbuttons clear briefcase"
            @click="parentInputType(3); clearTransaction();">Business Check</a></li>
          <li><a
            class="shopbuttons clear briefcase"
            @click="parentInputType(4); clearTransaction();">Business Deposit</a></li>
        </ul>

      </form>

    </div>
    <div
      v-show="readyToConfirm"
      id="transaction-confirm">
      <p class="heading">Confirm Transaction</p>
      <p>Please confirm that the transaction details are correct.</p>
      <div class="confirm clear">
        <div class="section">
          <span>
            <p>Account Number</p>
            <p id="confirmation">{{ account }}</p>
          </span>
          <span v-show="transactionType.includes('Check')">
            <p>Check Number</p>
            <p id="confirmation">{{ checkNumber }}</p>
          </span>
          <span v-show="amount">
            <p>Amount</p>
            <p id="confirmation">${{ parseFloat(amount).toFixed(2) }}</p>
          </span>
        </div>
        <div class="section">
          <span class="button">
            <a
              ref="confirmed"
              href=""
              class="confirmTransaction button check"
              @click.prevent="postTransaction">Confirm
            </a>
          </span>
        </div>
      </div>
      <p>If the transaction details above are not correct, please go back and <a
        class="nextButtons"
        @click="returnToTransaction">edit the transaction.</a>.</p>
    </div>
  </div>
</template>

<script>

export default {
  props : {
      transactionType: String,
  },
  data(){
    return {
      account:null,
      checkNumber:null,
      amount : null,
      // controls the invocation of the .focus event for the inputs
      submitted: false,
      readyToConfirm: false,
    };
  },
    mounted: function() {
      this.$refs.accountNumber.focus();
    },
    updated: function() {
      if(this.submitted) {
        this.$refs.accountNumber.focus();
        this.submitted = false;
      }
    },
  methods : {
    parentInputType(tab){
      this.$emit("update", tab);
    },
    tranactionTypeConverter(type){
      switch(type){
        case "Personal Check":
          return "check";
        case "Business Check":
          return "check";
        case "Personal Deposit":
          return "deposit";
        case "Business Deposit":
          return "deposit";
      }
    },
    tranactionTypeNameGenerator(type){
      switch(type){
        case "Personal Check":
          return "Check number #" + this.checkNumber;
        case "Business Check":
          return "Check number #" + this.checkNumber;
        case "Personal Deposit":
          return "Bank deposit #";
        case "Business Deposit":
          return "Bank deposit #";
      }
    },
    transactionTypeFromGenerator(type){
      switch(type){
        case "Personal Check":
          return this.account;
        case "Business Check":
          return this.account;
        case "Personal Deposit":
          return 0;
        case "Business Deposit":
          return 0;
      }
    },
    transactionTypeToGenerator(type){
      switch(type){
        case "Personal Check":
          return 0;
        case "Business Check":
          return 0;
        case "Personal Deposit":
          return this.account;
        case "Business Deposit":
          return this.account;
      }
    },
    clearTransaction() {
      this.submitted = true;
      this.readyToConfirm = false;
      this.account = null;
      this.checkNumber = null;
      this.amount = null;
    },
    returnToTransaction() {
      this.readyToConfirm = false;
      setTimeout(() => {
        this.$refs.accountNumber.focus();
      }, 100);
    },
    validateTransactionInfo() {
      if (this.account.length !== 3) {
        this.account = null;
        app.Message("please reenter the account number.", "warn");
        this.$refs.accountNumber.focus();
      }
      if (this.account.length === 3) {
        const isBusinessAccount = this.account.match(new RegExp(/^3/));
        if(isBusinessAccount && this.transactionType.includes("Personal")) {
          app.Message("Please enter an individual account number", "warn");
          this.$refs.accountNumber.focus();
          this.account = null;
        } else if (!isBusinessAccount && this.transactionType.includes("Business")) {
          app.Message("Please enter a business account number", "warn");
          this.$refs.accountNumber.focus();
          this.account = null;
        } else if (!this.checkNumber && !this.transactionType.includes("Deposit")) {
          app.Message("Please reenter the check number.", "warn");
          this.checkNumber = null;
          this.$refs.checkNumber.focus();
        } else if (!this.amount) {
          app.Message("Please enter an amount.", "warn");
          this.$refs.amount.focus();
        } else {
          this.readyToConfirm = true;
          //TODO: refactor this race condition
          setTimeout(() => {
            this.$refs.confirmed.focus();
          }, 100);
        }
      }
    },
    postTransaction(){
      var data = {
        "to":parseInt(this.transactionTypeToGenerator(this.transactionType)),
        "from":parseInt(this.transactionTypeFromGenerator(this.transactionType)),
        "checkNumber":parseInt(this.checkNumber),
        "amount":parseFloat(this.amount),
        "transactionType" : this.tranactionTypeConverter(this.transactionType),
        "session":app.session.data.id,
        "name" : this.tranactionTypeNameGenerator(this.transactionType)
      };
        app.api.Post( "/bank/transaction", data)
          .then( response => {
            if(response.status == "success"){
              app.Message("Transaction success", "success" );
              this.clearTransaction();
              this.$refs.accountNumber.focus();
            }
            else if (response.status == "bad check"){
              app.Message("Sorry, that check number is already in use", "warn" );
              this.readyToConfirm = false;
              this.checkNumber = null;
              //TODO: refactor this race condition here,
              setTimeout(() => {
                this.$refs.checkNumber.focus();
              }, 100);
            }
            else if (response.status == "bad account"){
              app.Message("Sorry, that account number wasn't found. Please try another.", "warn" );
              this.readyToConfirm = false;
              this.account = null;
              setTimeout(() => {
                this.$refs.accountNumber.focus();
              }, 100);
            }
          })
          .catch( x => app.Message( x.error, "error" ) );
      }
    },
};
</script>
